#ifndef __LIB_IO_H
#define __LIB_IO_H
#include <asm/types.h>

struct timespec;
struct sockaddr;
struct iovec;
struct iocb;

typedef struct io_context *io_context_t;

typedef enum io_iocb_cmd {

	IO_CMD_PREAD,
	IO_CMD_PWRITE,
	IO_CMD_READX,
	IO_CMD_ACCEPT,
	IO_CMD_CONNECT,
	IO_CMD_SENDTO,
	IO_CMD_RECVFROM,

	IO_CMD_POLL,
} io_iocb_cmd_t;

#if defined(__i386__) /* || little endian, 32 bits */
struct io_iocb_sendto {
	void	*msg;
	int	len;
	int	flags;
	struct sockaddr	*addr;
};

struct io_iocb_poll {
	int	events;
};	/* result code is the set of result flags or -'ve errno */

struct io_iocb_sockaddr {
	struct sockaddr *addr;
	int		len;
};	/* result code is the length of the sockaddr, or -'ve errno */

struct io_iocb_common {
	void		*buf;
	unsigned	__pad1;
	long		nbytes;
	unsigned	__pad2;
	long long	offset;
	long long	__pad3, __pad4;
};	/* result code is the amount read or -'ve errno */

struct io_iocb_vector {
	const struct iovec	*vec;
	int			nr;
	long long		offset;
};	/* result code is the amount read or -'ve errno */

struct iocb {
	void		*data;		/* Return in the io completion event */
	unsigned	__pad1;
	unsigned	key;		/* For use in identifying io requests */
	unsigned	__pad2;

	short		aio_io_opcode;	
	short		aio_reqprio;
	int		aio_fildes;

	union {
		struct io_iocb_common		c;
		struct io_iocb_vector		v;
		struct io_iocb_poll		poll;
		struct io_iocb_sockaddr	saddr;
	} u;
};

struct io_event {
	unsigned	data;
	unsigned	__pad1;
	unsigned	obj;
	unsigned	__pad2;
	unsigned	res;
	unsigned	__pad3;
	unsigned	res2;
	unsigned	__pad4;
};
#endif


typedef void (*io_callback_t)(io_context_t ctx, struct iocb *iocb, long res, long res2);

extern long io_queue_init(int maxevents, io_context_t *ctxp);
extern long io_queue_grow(io_context_t ctx, int new_maxevents);
extern long io_queue_release(io_context_t ctx);
extern long io_queue_wait(io_context_t ctx, struct timespec *timeout);
extern long io_queue_run(io_context_t ctx);
extern long io_submit(io_context_t ctx, int nr, struct iocb *ios[]);

static inline void io_set_callback(struct iocb *iocb, io_callback_t cb)
{
	iocb->data = cb;
}

static inline void io_prep_accept(struct iocb *iocb, int s, struct sockaddr *addr, int addrlen)
{
	iocb->aio_fildes = s;
	iocb->aio_io_opcode = IO_CMD_ACCEPT;
	iocb->aio_reqprio = 0;
	iocb->u.c.buf = addr;
	iocb->u.c.nbytes = addrlen;
	iocb->u.c.offset = 0;
}

static inline void io_prep_pread(struct iocb *iocb, int fd, void *buf, long count, long long offset)
{
	iocb->aio_fildes = fd;
	iocb->aio_io_opcode = IO_CMD_PREAD;
	iocb->aio_reqprio = 0;
	iocb->u.c.buf = buf;
	iocb->u.c.nbytes = count;
	iocb->u.c.offset = offset;
}

static inline void io_prep_pwrite(struct iocb *iocb, int fd, void *buf, long count, long long offset)
{
	iocb->aio_fildes = fd;
	iocb->aio_io_opcode = IO_CMD_PWRITE;
	iocb->aio_reqprio = 0;
	iocb->u.c.buf = buf;
	iocb->u.c.nbytes = count;
	iocb->u.c.offset = offset;
}

static inline void io_prep_poll(struct iocb *iocb, io_callback_t cb, int fd, int events)
{
	iocb->data = cb;
	iocb->aio_fildes = fd;
	iocb->aio_io_opcode = IO_CMD_POLL;
	iocb->aio_reqprio = 0;
	iocb->u.poll.events = events;
}

static inline long io_poll(io_context_t ctx, struct iocb *iocb, io_callback_t cb, int fd, int events)
{
	io_prep_poll(iocb, cb, fd, events);
	return io_submit(ctx, 1, &iocb);
}

#endif
